// -*- C++ -*-
//
// PDF.h is a part of ThePEG - Toolkit for HEP Event Generation
// Copyright (C) 1999-2007 Leif Lonnblad
//
// ThePEG is licenced under version 2 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
#ifndef ThePEG_PDF_H
#define ThePEG_PDF_H
// This is the declaration of the PDF class.

#include "ThePEG/PDF/PartonBinInstance.h"
// #include "PDF.fh"
// #include "PDF.xh"

namespace ThePEG {

/**
 * PDF is a simple wrapper class with normal copy-semantics which
 * holds a PDFBase object and a ParticleData object for which to
 * determine parton densities.
 */
class PDF {

public:

  /** @name Standard constructors, assignment and destructors. */
  //@{
  /**
   * Default constructor.
   */
  inline PDF();

  /**
   * Constructor from a given PartonBinInstance.
   */
  inline PDF(tcPBIPtr);

  /**
   * Constructor from a given PDFBase and ParticleData object.
   */
  inline PDF(tcPDFPtr, tcPDPtr);
  //@}

public:

  /** @name Access the parton densities. */
  //@{
  /**
   * Return the density for the given \a parton, for a given \a
   * partonScale and logarithmic momentum fraction \a l assuming the
   * particle has a virtuality \a particleScale.
   */
  inline double xfl(tcPPtr parton, Energy2 partonScale, double l,
		    Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the density for the given \a parton, for a given \a
   * partonScale and momentum fraction \a x assuming the
   * particle has a virtuality \a particleScale.
   */
  inline double xfx(tcPPtr parton, Energy2 partonScale, double x,
		    double eps = 0.0, Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the valence density for the given \a parton, for a given
   * \a partonScale and logarithmic momentum fraction \a l assuming
   * the particle has a virtuality \a particleScale.
   */
  inline double xfvl(tcPPtr parton, Energy2 partonScale, double l,
		     Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the valence density for the given \a parton, for a given
   * \a partonScale and momentum fraction \a x assuming the particle
   * has a virtuality \a particleScale.
   */
  inline double xfvx(tcPPtr parton, Energy2 partonScale, double x,
		     double eps = 0.0, Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the density for the given \a parton, for a given \a
   * partonScale and logarithmic momentum fraction \a l assuming the
   * particle has a virtuality \a particleScale.
   */
  inline double xfl(tcPDPtr parton, Energy2 partonScale, double l,
		    Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the density for the given \a parton, for a given \a
   * partonScale and momentum fraction \a x assuming the
   * particle has a virtuality \a particleScale.
   */
  inline double xfx(tcPDPtr parton, Energy2 partonScale, double x,
		    double eps = 0.0, Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the valence density for the given \a parton, for a given
   * \a partonScale and logarithmic momentum fraction \a l assuming
   * the particle has a virtuality \a particleScale.
   */
  inline double xfvl(tcPDPtr parton, Energy2 partonScale, double l,
		     Energy2 particleScale = 0.0*GeV2) const;

  /**
   * Return the valence density for the given \a parton, for a given
   * \a partonScale and momentum fraction \a x assuming the particle
   * has a virtuality \a particleScale.
   */
  inline double xfvx(tcPDPtr parton, Energy2 partonScale, double x,
		     double eps = 0.0, Energy2 particleScale = 0.0*GeV2) const;
  //@}

  
  /**
   * The parton density object.
   */
  inline tcPDFPtr pdf() const;

  /**
   * The particle for which the parton density is used.
   */
  inline tcPDPtr particle() const;

private:

  /**
   * The parton density object.
   */
  tcPDFPtr thePDF;

  /**
   * The particle for which the parton density is used.
   */
  tcPDPtr theParticle;

};

}

#include "PDF.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "PDF.tcc"
#endif

#endif /* ThePEG_PDF_H */
